/*
 * Test directory reading operations.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/sysmacros.h>  /* major, minor */

#include <fsclient/client.h>
#include <systypes/fcntl.h>
#include <systypes/format.h>

#define FMT64d "%" pFMT64 "d"
#define FMTnlinkd "%" pFMTnlink "d"



int main(int argc, char *argv[])
{
  if (argc < 3)
  {
    printf("Need a directory name and an expected number of entries.\n");
    return 1;
  }

  char *filename = argv[1];
  int entries = atoi(argv[2]);

  printf("Opening %s...\n", filename);

  file_t *reader = client_opendir(filename);

  if (!client_opened(reader))
  {
    client_close(reader);
    printf("Could not read from directory.\n");
    return 1;
  }

  printf("Reading...\n");

  struct dirent *dirent;
  int found = 0;

  while ((dirent = client_readdir(reader)) != NULL)
  {
    char path[strlen(filename) + 1 + strlen(dirent->d_name) + 1];
    struct stat st;

    printf("> %s\n", dirent->d_name);

    /* Obtain file metadata. */

    sprintf(path, "%s/%s", filename, dirent->d_name);

    if (client_stat(path, &st))
    {
      client_close(reader);
      printf("Could not obtain metadata for file: %s\n", dirent->d_name);
      return 1;
    }

    /* NOTE: From e2access... */

    /* Terse stat output:
       %n %s %b %f %u %g %D %i %h %t %T %X %Y %Z %W %o %C */

    printf("%s %ld %ld %x %d %d ",
        path, st.st_size, st.st_blocks, st.st_mode, st.st_uid, st.st_gid);

    printf(FMT64d " %ld " FMTnlinkd " %x %x ",
        st.st_dev, st.st_ino, st.st_nlink,
        major(st.st_rdev), minor(st.st_rdev));

    printf("%ld %ld %ld ",
        st.st_atim.tv_sec, st.st_mtim.tv_sec, st.st_ctim.tv_sec);

    /* NOTE: Arbitrary values:
       %W (creation time) given as 0
       %o (I/O transfer size hint) given as 0
       %C (SELinux security context) given as empty string */

    printf("%d %d %s\n",
        0, 0, "");

    free(dirent);
    found++;
  }

  printf("Directory shown.\n");
  printf("Found expected number of entries: %s\n", found == entries ? "True" : "False");

  /* Open again, reading a single entry only. */

  reader = client_opendir(filename);

  if (!client_opened(reader))
  {
    client_close(reader);
    printf("Could not read from directory.\n");
    return 1;
  }

  dirent = client_readdir(reader);

  if (dirent != NULL)
  {
    printf("> %s\n", dirent->d_name);
    free(dirent);
  }

  printf("Entry shown.\n");

  printf("End of test.\n");
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
